import { Node, Variant } from 'godot'
import { export_ } from 'godot.annotations'

export interface Damageable {
  apply_damage(source: any, amount: number): void
}

export interface Recoverable {
  apply_recovery(source: any, amount: number): void
}

export default class Health extends Node implements Damageable, Recoverable {
  @export_(Variant.Type.TYPE_FLOAT)
  private _max_health: number = 100

  @export_(Variant.Type.TYPE_FLOAT)
  private _current_health: number = 0

  _ready() {
    this._current_health = this._max_health
  }

  apply_damage(_source: any, amount: number): void {
    this._current_health = Math.max(0, this._current_health - amount)
  }

  apply_recovery(_source: any, amount: number): void {
    this._current_health = Math.min(this._max_health, this._current_health + amount)
  }
}

