import { GArray, Node, Object, StringName } from 'godot'

interface Predicate<T> {
  (value: T): boolean
}

export function queue_free_children<T extends Node>(node: T) {
  const children: GArray<Node> = node.get_children()
  for (const child of children) {
    child.queue_free()
  }
}

type RequiredField<T, Field extends keyof T> = Omit<T, Field> & Required<Pick<T, Field>>

export function has_method(name: StringName, obj: Object): boolean {
  return obj.has_method(name)
}

export function implements_interface<T>(method_names: StringName[], obj: Object): obj is Object & T {
  return method_names.every(name => has_method(name, obj))
}

export function find_in_anscestors<T extends Node>(root: T | null | undefined, predicate: Predicate<Node>, height: number = Infinity): T | null | undefined {
  if (height < 0 || root == null) {
    return
  }

  if (predicate(root)) {
    return root
  }

  return find_in_anscestors(root.get_parent(), predicate, height - 1) as T
}

export function find_in_descendents<T extends Node>(root: T, predicate: Predicate<Node>, depth: number = Infinity): T | null | undefined {
  if (depth < 0) {
    return
  }

  if (predicate(root)) {
    return root
  }

  const count = root.get_child_count()
  for (let i = 0; i < count; i++) {
    const child = root.get_child(i)
    const found = find_in_descendents(child, predicate, depth - 1)

    if (found != null) {
      return found as T
    }
  }
}
