import { AnimationTree, Expression, float64, GArray, GError, Node } from 'godot'
import Player from './player'
import PlayerInput from './player_input'
import { export_expression, onready } from './annotations'

export const PlayerAnimationName = Object.freeze({
  Idle: 'Idle',
  Walk: 'Walk',
  Run: 'Run',
  Interact: 'Interact',
  Aim: 'PistolIdle',
  Fire: 'Fire'
} as const)

const expr = (src: string) => {
  if (!src) {
    return
  }

  const expr = new Expression()
  const err = expr.parse(src)

  if (err !== GError.OK) {
    throw new Error(`Unable to parse expression: ${expr.get_error_text()}`)
  }

  return expr
}

const empty = new GArray()
export default class PlayerAnimation extends AnimationTree {
  //private static Interact = 'parameters/conditions/interact'
  //private static Movement = 'parameters/Movement/blend_position'
  private static Movement = ['parameters/Unarmed/blend_position', 'parameters/Handgun/Movement/BlendSpace1D/blend_position']

  @export_expression()
  readonly fire_expression: string = ''

  @export_expression()
  readonly move_speed_expression: string = ''

  @onready('../')
  readonly player!: Player

  @onready('../Input')
  readonly player_input!: PlayerInput

  @onready(n => n.get_node(n.advance_expression_base_node))
  private advance_node!: Node

  private _fire_expr?: Expression
  private _move_expr?: Expression

  _ready(): void {
    this.advance_node = this.get_node(this.advance_expression_base_node)
    this._fire_expr = expr(this.fire_expression)
    this._move_expr = expr(this.move_speed_expression)
  }

  _physics_process(_delta: float64): void {
    //this.set(PlayerAnimation.Movement, this.player.velocity.length_squared())
    const move_result = this._move_expr?.execute(empty, this.advance_node, true)

    if (this._move_expr?.has_execute_failed()) {
      console.error(this._move_expr.get_error_text())
    }

    PlayerAnimation.Movement.forEach(key => {
      this.set(key, move_result)
    })
    //this.set(PlayerAnimation.Movement, this._move_expr.execute())
  }
}
