import { GError, Node, Resource, ResourceLoader } from 'godot'

export default class AsyncResourceLoader extends Node {
  private static _instance: AsyncResourceLoader

  static get instance() {
    return this._instance
  }

  _ready(): void {
    AsyncResourceLoader._instance = this
  }

  load<T extends Resource>(path: string, hint_string: string = ''): Promise<T> {
    return new Promise(async (resolve, reject) => {
      const err = ResourceLoader.load_threaded_request(path, hint_string)

      if (err != GError.OK) {
        return reject(new Error(`failed to load ${path}`))
      }

      const get_status = () => ResourceLoader.load_threaded_get_status(path)

      let status = get_status()
      while (status === ResourceLoader.ThreadLoadStatus.THREAD_LOAD_IN_PROGRESS) {
        await this.get_tree().process_frame.as_promise()
        status = get_status()
      }

      if (get_status() === ResourceLoader.ThreadLoadStatus.THREAD_LOAD_LOADED) {
        return resolve(ResourceLoader.load_threaded_get(path) as T)
      } else {
        return reject(new Error(`failed to load ${path}`))
      }
    })
  }
}

